#!/usr/bin/env python3.4
# -*- coding: utf-8 -*-
#
#  SenseHat-20.py	 10 Avril 2017
#  
#  Copyright 2017  <pi@CDBI-PI3>
#
#Utilise Boitier PI3-Version 2
#BOUTONS PIFACE
#0	Arret
#1	Affiche température + BareGraphe
#2	Affiche Les 3 Mesures
#3	Affiche Température CPU
#4	Lance/Stoppe l'enregistrement dans un fichier MesuSHxxxx

#On ne peut pas stopper la matrice pendant un message

import os
import sys
import subprocess
from sense_hat import SenseHat
from time import sleep
import pifacedigitalio as pfio
from datetime import datetime
import RPi.GPIO as GPIO

#===================================================================
def run_cmd(cmd):
	return subprocess.check_output(cmd, shell=True).decode('utf-8')

#===================================================================
def BareG(cpt,Reso):
	sense.clear()
	#Affiche la liste TabT qui est en 1/10 de °C
	#Recherche du Max et ajustement du Min
	#Reso=0 Dynamique de 4°c  (8 Pixels de 0.5°C)
	#Reso=1 Dynamique de 8°C  (8 Pixels de 1°C)

	tmax=0
	if Reso == 0:	#Pas de 0.5°
		Div=1
		for x in range (0,len(tabT)):
			tempe=tabT[x]
			if tmax < tempe+4:
				tmax=tempe+4
		tmin=tmax-8
		if tmin < 2:
			tmax=tmax+2
			tmin=tmax-8
		sense.show_message(str(tmax/10),text_colour=[100,0,80])

	else:	#Pas de 1°
		Div=10
		for x in range (0,len(tabT)):
			tempe=int(tabT[x]/10)
			if tmax < tempe+2:
				tmax=tempe+2
			tmin=tmax-8
			if tmin < 2:
				tmax=tmax+2
				tmin=tmax-8
		sense.show_message(str(int(tmax)),text_colour=[100,0,80])

	#Affichage
	for x in range (0,8):
		if x == (cpt+1):
			break
		tempe=int((tabT[x]/Div)-tmin)
		if (tempe >= 0) and (tempe < 9):
			for y in range (0,tempe):
				sense.set_pixel(x,y,100,0,80)
			for y in range (tempe,8):
				sense.set_pixel(x,y,50,50,50)
		else:
			for y in range (0,8):
				sense.set_pixel(x,y,0,0,0)

	sense.flip_v()
	sleep(3)

#===================================================================
def SW_pressed(event):
	numB=event.pin_num
	Buz.start(10)
#===================================================================
def SW_unpressed(event):
	global Mesu
	global NomFich
	global flagENR

	Buz.stop()

	if event.pin_num == 4:
		if (flagENR == False):
			#Création du fichier
			NomFich=Chemin + "MesuSH" + datetime.now().strftime('_%d%m-%H%M') + ".txt"
			nF=open(NomFich,"w")
			nF.write("RELEVE CLIMATIQUE-SENSEHAT  CDBI-PI3\r\n")
			nF.write (datetime.now().strftime('Ouverture: %d-%m-%Y  %H:%M:%S') +'\r\n')
			nF.write("=============================================\r\n")
			nF.close()
			print ("Ouverture de " + NomFich)
			flagENR=True
		elif (flagENR == True):
			#Fermeture du Fichier
			nF=open(NomFich,"a")
			nF.write("=============================================\r\n")
			nF.write(datetime.now().strftime('Fermeture: %d-%m-%Y  %H:%M:%S'))
			nF.close()
			print ("Fermeture de " + NomFich)
			flagENR=False

	else:
		Mesu=event.pin_num

#===================================================================
def Rota():
	x,y,z = sense.get_accelerometer_raw().values()
	x=round(x,0)
	y=round(y,0)
	z=round(z,0)
	if x == 1:
		a=0
	elif x == -1:
		a=180
	elif y == 1:
		a=90
	elif y == -1:
		a=270
	else:
		a=0
	sense.set_rotation(a)
	#print (str(x) + '  ' + str(y))

#===================================================================
def get_CPUtemp():
	return run_cmd(GET_TEMP_CMD)

#===================================================================
def Fermer(SD):
	flagRUN=False
	sense.show_letter("A",text_colour=[200,100,100])
	sleep(2)
	sense.clear()
	listener.deactivate()
	pfio.deinit()
	#Si Fichier en cours  Enregistre la Date de Fermeture
	if flagENR == True:
		nF=open(NomFich,"a")
		nF.write("=============================================\r\n")
		nF.write(datetime.now().strftime('Fermeture sur Arret: %d-%m-%Y  %H:%M:%S'))
		nF.close()

	sense.low_light=False
	if SD == True:
		print ("\n << Arret et ShutDown >>")
		sleep (2)
		os.system("sudo halt")
	else:
		print ("\n << ARRET >>")
		sys.exit(0)

#===================================================================
def Existe(f):
	try:
		nF=open(f,'r')
		nF.close()
		return 1
	except:
		return 0

#==========================MAIN=====================================
pfio.init()
pfd=pfio.PiFaceDigital()
listener=pfio.InputEventListener(chip=pfd)
for i in range(6):
	listener.register(i,pfio.IODIR_ON,SW_pressed)
	listener.register(i,pfio.IODIR_OFF,SW_unpressed)
listener.activate()
#On met le GPIO18 en sortie et PWM pour le Buzzer
#disable warnings car déjà utilisé par le ShutDown
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(18,GPIO.OUT)
Buz=GPIO.PWM(18,600)

#Commande pour lire la Temp du CPU
GET_TEMP_CMD = "/opt/vc/bin/vcgencmd measure_temp"

Chemin=""			#Dossier des fichiers
tabT=[]				#Liste des 8 dernières Températures au 1/10 °C
cpt=0				#Compteur de 0 à 7 pour le BarGraphe
BareGreso=1			#0=pas de 0.5°  1=pas de 1°C
Mesu=-1				#Type de Mesures renvoyé par les boutons
Vit=0.15			#Vitesse de défilement de la Matrice
Tlapse1=2			#Temps entre chaque cycle de mesure
SenseLight=0		#1 pour intensité moitié
corT=0				#correction Température pour compenser le boitier
flagENR=False		#Enregistrement ou non
flagRUN=False		#Boucle while principale

fichier="ConfigSH.txt"
if Existe(fichier):
	nF=open(fichier,'r')
	BareGreso=nF.readline()
	BareGreso=int(nF.readline())
	Vit=nF.readline()
	Vit=float(nF.readline())
	Tlapse1=nF.readline()
	Tlapse1=int(nF.readline())
	SenseLight=nF.readline()
	SenseLight=int(nF.readline())
	r=nF.readline()
	r=nF.readline()
	Chemin=r[0:len(r)-1]  #enlève le saut de ligne
	corT=nF.readline()
	corT=float(nF.readline())
	nF.close()

sense = SenseHat()
sense.clear()
sense.low_light=SenseLight
#initialise l'orientation
sense.set_rotation(0)
Rota()

while (Mesu == -1):
	sense.show_message("CDBI ?",scroll_speed=0.2,text_colour=[200,200,0],back_colour=[0,50,0])
	#On peut changer l'Orientation durant cette phase
	Rota()

if Mesu == 0:
	Tlapse2=0
	Fermer(False)
else:
	
	flagRUN=True
	Tlapse2=Tlapse1

sense.show_letter(str(Mesu),text_colour=[0,0,150],back_colour=[0,0,0])
sleep(2)

while (flagRUN):
	try:
		t=sense.get_temperature()
		t=t+corT
		#on affiche le 1/10 ° pour le Baregraphe
		tabT.append (round(t,1)*10)
		h=sense.get_humidity()
		p=sense.get_pressure()
		t='T='+str(round(t,1))
		h='H='+str(round(h,1))
		p='P='+str(round(p,1))

		if flagENR == True:
			nF=open(NomFich,"a")
			Timestamp=datetime.now().strftime('%d-%m-%Y_%H:%M:%S  ')
			L=Timestamp + t + '  ' + h + '  ' + p
			print (L)
			nF.write(L + "\r\n")	#Ajoute un CRLF pour lecture sous Windows
			nF.close()

		if Mesu == 0:
			Fermer(False)
			break

		elif Mesu == 1:
			#Température + BareGraphe
			sense.show_message(t,scroll_speed=Vit,text_colour=[180,180,0],back_colour=[50,0,0])
			if Mesu == 0:
				Fermer(False)
				break
			BareG(cpt,BareGreso)
			cpt +=1
			if cpt > 7:
				cpt=0
				tabT=[]

		elif Mesu == 2:
			#Température+Humidité+Pression
			sense.show_message(t,scroll_speed=Vit,text_colour=[180,180,0],back_colour=[50,0,0])
			if Mesu == 0:
				Fermer(False)
				break
			sense.show_message(h,scroll_speed=Vit,text_colour=[180,180,0],back_colour=[0,0,50])
			if Mesu == 0:
				Fermer(False)
				break
			sense.show_message(p,scroll_speed=Vit,text_colour=[200,200,0],back_colour=[0,50,0])

		elif Mesu == 3:
			#Température CPU
			msg=get_CPUtemp()
			sense.low_light=True
			Tlapse2=0
			sense.show_message(msg,scroll_speed=Vit,text_colour=[100,0,0],back_colour=[0,0,0])
			if Mesu == 0:
				Fermer(False)
				break

		elif Mesu == 5:
			#Arret et ShutDown
			Fermer(True)

		if flagENR == True:
			sense.show_letter("F",text_colour=[50,50,50])
		else:
			sense.show_letter("W",text_colour=[50,50,50])
		
		if Mesu != 0:
			sleep(Tlapse2)

	except (KeyboardInterrupt):
		flagRUN=False
		sense.set_rotation(0)
		Fermer()
		
